use serenity::{prelude::GatewayIntents, Client as SerenityClient};

use crate::{database::Client as DatabaseClient, environment::get_env_variable};

use crate::discord::event_handler::Handler;

pub struct Client {
    serenity_client: SerenityClient,
}

impl Client {
    pub async fn new() -> Self {
        let mut database_client = DatabaseClient::new();
        database_client.connect().await;

        let discord_token = get_env_variable("DISCORD_TOKEN");
        let intents = GatewayIntents::GUILD_MESSAGES | GatewayIntents::MESSAGE_CONTENT;

        let event_handler = Handler {
            database: database_client,
        };

        let serenity_client = match SerenityClient::builder(discord_token, intents)
            .event_handler(event_handler)
            .await
        {
            Ok(c) => c,
            Err(e) => panic!("Failed to instantiate Discord Client: {e}"),
        };

        Client { serenity_client }
    }

    pub async fn start(&mut self) {
        if let Err(e) = self.serenity_client.start().await {
            panic!("Could not connect the bot: {e}");
        }
    }
}
