use mongodb::bson::doc;

use serenity::{
    builder::{CreateApplicationCommand, CreateInteractionResponseData},
    model::{
        application::interaction::application_command::ApplicationCommandInteraction,
        prelude::{
            command::CommandOptionType, interaction::application_command::CommandDataOptionValue,
        },
    },
};

use crate::database::{models::Tag, Client as DatabaseClient};

pub fn register(command: &mut CreateApplicationCommand) -> &mut CreateApplicationCommand {
    command
        .name("create_tag")
        .description("Add a new tag")
        .create_option(|option| {
            option
                .name("tag")
                .description("The tag to create")
                .kind(CommandOptionType::String)
                .required(true)
        })
}

pub async fn run<'a, 'b>(
    response: &'a mut CreateInteractionResponseData<'b>,
    command: &ApplicationCommandInteraction,
    database: &DatabaseClient,
) -> &'a mut CreateInteractionResponseData<'b> {
    let arg = command
        .data
        .options
        .get(0)
        .expect("Missing option")
        .resolved
        .as_ref()
        .expect("Could not deserialize option");

    let guild_id = command
        .guild_id
        .expect("Could not fetch guild id")
        .to_string();

    if let CommandDataOptionValue::String(tag_name) = arg {
        let matching_tags = database
            .get_all::<Tag>(Some(doc! {"name": tag_name, "guild_id": guild_id.as_str()}))
            .await;

        if !matching_tags.is_empty() {
            response.content("This tag already exist for this server");
        } else {
            match database
                .insert_one(Tag {
                    id: None,
                    name: tag_name.to_string(),
                    guild_id: guild_id.clone(),
                    is_nsfw: false,
                    subscribers: vec![],
                })
                .await
            {
                Ok(_) => response.content("Tag successfully created."),
                Err(_) => response.content("Error creating the tag"),
            };
        }
    }

    response
}
