use crate::database::Client as DatabaseClient;
use serenity::{
    async_trait,
    model::gateway::Ready,
    model::{
        application::command::Command,
        prelude::{
            interaction::{Interaction, InteractionResponseType},
            ResumedEvent,
        },
    },
    prelude::{Context, EventHandler},
};

use super::commands::*;

const MAX_ARGS_NUMBER: u32 = 25;

pub struct Handler {
    pub database: DatabaseClient,
}

#[async_trait]
impl EventHandler for Handler {
    async fn ready(&self, ctx: Context, ready: Ready) {
        println!("Successfully connected as {}", ready.user.name);

        match Command::set_global_application_commands(&ctx.http, |commands| {
            commands
                .create_application_command(|command| source_code::register(command))
                .create_application_command(|command| create_tag::register(command))
                .create_application_command(|command| {
                    bulk_create_tag::register(command, MAX_ARGS_NUMBER)
                })
                .create_application_command(|command| delete_tag::register(command))
                .create_application_command(|command| list_tags::register(command))
        })
        .await
        {
            Ok(_) => println!("Successfully registered application commands"),
            Err(e) => println!("Failed to register application commands: {e}"),
        };
    }

    async fn resume(&self, _: Context, _: ResumedEvent) {
        println!("Successfully reconnected.")
    }

    async fn interaction_create(&self, ctx: Context, interaction: Interaction) {
        if let Interaction::ApplicationCommand(command) = interaction {
            println!("Received command {}", command.data.name);

            if let Err(e) = command
                .create_interaction_response(&ctx.http, |response| {
                    response
                        .kind(InteractionResponseType::ChannelMessageWithSource)
                        .interaction_response_data(|message| match command.data.name.as_str() {
                            "source_code" => source_code::run(message),
                            "create_tag" => create_tag::run(message, &command, &self.database),
                            _ => message.content("Not yet implemented"),
                        })
                })
                .await
            {
                println!("Failed to answer to command: {e}");
            }
        }
    }
}
